import { createGridComponent } from "./createGridComponent";

export const Grid = createGridComponent({
  getColumnWidth: ({ columnWidth }, index) => columnWidth,
  getRowHeight: ({ rowHeight }, index) => rowHeight,
  getEstimatedTotalHeight: ({ rowCount, rowHeight }) => rowHeight * rowCount,
  getEstimatedTotalWidth: ({ columnCount, columnWidth }) =>
    columnWidth * columnCount,
  getOffsetForColumn: (
    { columnWidth, width },
    columnIndex,
    align,
    scrollLeft
  ) => {
    const maxOffset = columnIndex * columnWidth;
    const minOffset = columnIndex * columnWidth - width + columnWidth;

    switch (align) {
      case "start":
        return maxOffset;
      case "end":
        return minOffset;
      case "center":
        return Math.round(minOffset + (maxOffset - minOffset) / 2);
      case "auto":
      default:
        if (scrollLeft >= minOffset && scrollLeft <= maxOffset) {
          return scrollLeft;
        } else if (scrollLeft - minOffset < maxOffset - scrollLeft) {
          return minOffset;
        } else {
          return maxOffset;
        }
    }
  },
  getOffsetForRow: ({ rowHeight, height }, rowIndex, align, scrollTop) => {
    const maxOffset = rowIndex * rowHeight;
    const minOffset = rowIndex * rowHeight - height + rowHeight;

    switch (align) {
      case "start":
        return maxOffset;
      case "end":
        return minOffset;
      case "center":
        return Math.round(minOffset + (maxOffset - minOffset) / 2);
      case "auto":
      default:
        if (scrollTop >= minOffset && scrollTop <= maxOffset) {
          return scrollTop;
        } else if (scrollTop - minOffset < maxOffset - scrollTop) {
          return minOffset;
        } else {
          return maxOffset;
        }
    }
  },
  getColumnStartIndexForOffset: ({ columnWidth, columnCount }, offset) =>
    Math.min(columnCount - 1, Math.floor(offset / columnWidth)),
  getColumnStopIndexForStartIndex: (
    { columnWidth, columnCount, width },
    startIndex
  ) => {
    return Math.min(
      columnCount - 1,
      Math.round(startIndex + width / columnWidth)
    );
  },
  getRowStartIndexForOffset: ({ rowHeight, rowCount }, offset) =>
    Math.min(rowCount - 1, Math.floor(offset / rowHeight)),
  getRowStopIndexForStartIndex: (
    { rowHeight, rowCount, height },
    startIndex
  ) => {
    return Math.min(rowCount - 1, Math.round(startIndex + height / rowHeight));
  },
  validateProps: ({ columnWidth, rowHeight }) => {
    if (typeof columnWidth !== "number") {
      throw Error(
        'An invalid "columnWidth" prop has been specified. ' +
          "Value should be a number. " +
          `"${
            columnWidth === null ? "null" : typeof columnWidth
          }" was specified.`
      );
    }

    if (typeof rowHeight !== "number") {
      throw Error(
        'An invalid "rowHeight" prop has been specified. ' +
          "Value should be a number. " +
          `"${rowHeight === null ? "null" : typeof rowHeight}" was specified.`
      );
    }
  }
});
