import {createListComponent} from "./createListComponent";

export const List = createListComponent({
  getCellSize: ({ cellSize, size }, index) => cellSize,
  getEstimatedTotalSize: ({ cellSize, count }) => cellSize * count,
  getOffsetForIndex: ({ cellSize, direction, height, width }, index, align, scrollOffset) => {
    const maxOffset = index * cellSize;
    const minOffset = index * cellSize - (direction === "horizontal" ? width : height) + cellSize;

    switch (align) {
      case "start":
        return maxOffset;
      case "end":
        return minOffset;
      case "center":
        return Math.round(minOffset + (maxOffset - minOffset) / 2);
      case "auto":
      default:
        if (scrollOffset >= minOffset && scrollOffset <= maxOffset) {
          return scrollOffset;
        } else if (scrollOffset - minOffset < maxOffset - scrollOffset) {
          return minOffset;
        } else {
          return maxOffset;
        }
    }
  },
  getStartIndexForOffset: ({ cellSize, count }, offset) => Math.min(count - 1, Math.floor(offset / cellSize)),
  getStopIndexForStartIndex: ({ cellSize, count, direction, height, width }, startIndex) => {
    const size = direction === "horizontal" ? width : height;
    return Math.min(count - 1, Math.round(startIndex + size / cellSize));
  },
  validateProps: ({ cellSize }) => {
    if (typeof cellSize !== "number") {
      throw Error('An invalid "cellSize" prop has been specified. ' + "Value should be a number. " + `"${cellSize === null ? "null" : typeof cellSize}" was specified.`);
    }
  }
});
