var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import React from "react";

const IS_SCROLLING_DEBOUNCE_INTERVAL = 150;

export function createGridComponent({
  getColumnStartIndexForOffset,
  getColumnStopIndexForStartIndex,
  getColumnWidth,
  getEstimatedTotalHeight,
  getEstimatedTotalWidth,
  getOffsetForColumn,
  getOffsetForRow,
  getRowHeight,
  getRowStartIndexForOffset,
  getRowStopIndexForStartIndex,
  validateProps
}) {
  var _class, _temp2;

  return _temp2 = _class = class List extends React.Component {
    constructor(...args) {
      var _temp;

      return _temp = super(...args), this._cellStyleCache = {}, this._resetIsScrollingTimeoutId = null, this.state = {
        isScrolling: false,
        horizontalScrollDirection: "forward",
        scrollLeft: 0,
        scrollTop: 0,
        verticalScrollDirection: "forward"
      }, this.onScroll = event => {
        const { scrollLeft, scrollTop } = event.currentTarget;
        this.setState(prevState => ({
          isScrolling: true,
          horizontalScrollDirection: prevState.scrollLeft < scrollLeft ? "forward" : "backward",
          scrollLeft,
          scrollTop,
          verticalScrollDirection: prevState.scrollTop < scrollTop ? "forward" : "backward"
        }), this.resetIsScrollingDebounced);
      }, this.scrollingContainerRef = ref => {
        this._scrollingContainer = ref;
      }, this.resetIsScrollingDebounced = () => {
        if (this._resetIsScrollingTimeoutId !== null) {
          clearTimeout(this._resetIsScrollingTimeoutId);
        }

        this._resetIsScrollingTimeoutId = setTimeout(this.resetIsScrolling, IS_SCROLLING_DEBOUNCE_INTERVAL);
      }, this.resetIsScrolling = () => {
        this._resetIsScrollingTimeoutId = null;

        this.setState({ isScrolling: false }, () => {
          // Clear style cache after state update has been committed.
          // This way we don't break pure sCU for cells that don't use isScrolling param.
          this._cellStyleCache = {};
        });
      }, _temp;
    }

    static getDerivedStateFromProps(nextProps, prevState) {
      validateSharedProps(nextProps);
      validateProps(nextProps);
      return null;
    }

    scrollTo({
      scrollLeft,
      scrollTop
    }) {
      if (this._scrollingContainer != null) {
        this._scrollingContainer.scrollLeft = scrollLeft;
        this._scrollingContainer.scrollTop = scrollTop;
      }
    }

    scrollToCell({
      align = "auto",
      columnIndex,
      rowIndex
    }) {
      const { scrollLeft, scrollTop } = this.state;

      this.scrollTo({
        scrollLeft: getOffsetForColumn(this.props, columnIndex, align, scrollLeft),
        scrollTop: getOffsetForRow(this.props, rowIndex, align, scrollTop)
      });
    }

    componnetWillUnmount() {
      if (this._resetIsScrollingTimeoutId !== null) {
        clearTimeout(this._resetIsScrollingTimeoutId);
      }
    }

    render() {
      const { className, height, style, width } = this.props;
      const { isScrolling } = this.state;

      const estimatedTotalHeight = getEstimatedTotalHeight(this.props);
      const estimatedTotalWidth = getEstimatedTotalWidth(this.props);

      return React.createElement(
        "div",
        {
          className: className,
          ref: this.scrollingContainerRef,
          style: _extends({
            position: "relative",
            height,
            width,
            overflow: "auto"
          }, style),
          onScroll: this.onScroll
        },
        React.createElement(
          "div",
          {
            style: {
              height: estimatedTotalHeight,
              overflow: "hidden",
              pointerEvents: isScrolling ? "none" : "",
              width: estimatedTotalWidth
            }
          },
          this.renderCells()
        )
      );
    }

    renderCells() {
      const { children, useIsScrolling } = this.props;
      const { isScrolling } = this.state;

      const [columnStartIndex, columnStopIndex] = this.getHorizontalRangeToRender();
      const [rowStartIndex, rowStopIndex] = this.getVerticalRangeToRender();

      const cells = [];

      for (let rowIndex = rowStartIndex; rowIndex <= rowStopIndex; rowIndex++) {
        for (let columnIndex = columnStartIndex; columnIndex <= columnStopIndex; columnIndex++) {
          const key = `${rowIndex}:${columnIndex}`;

          // Cache cell styles while scrolling,
          // So that pure component sCU will prevent re-renders.
          let style;
          if (this._cellStyleCache.hasOwnProperty(key)) {
            style = this._cellStyleCache[key];
          } else {
            // TODO Get position of cell using helper, not hard-coded number type
            this._cellStyleCache[key] = style = {
              position: "absolute",
              left: columnIndex * getColumnWidth(this.props, columnIndex),
              top: rowIndex * getRowHeight(this.props, rowIndex),
              height: getRowHeight(this.props, rowIndex),
              width: getColumnWidth(this.props, columnIndex)
            };
          }

          cells.push(children({
            columnIndex,
            key,
            isScrolling: useIsScrolling ? isScrolling : undefined,
            rowIndex,
            style
          }));
        }
      }

      return cells;
    }

    getHorizontalRangeToRender() {
      const { columnCount, overscanCount } = this.props;
      const { horizontalScrollDirection, scrollLeft } = this.state;

      const startIndex = getColumnStartIndexForOffset(this.props, scrollLeft);
      const stopIndex = getColumnStopIndexForStartIndex(this.props, startIndex);

      // Overscan by one cell in each direction so that tab/focus works.
      // If there isn't at least one extra cell, tab loops back around.
      const overscanBackward = horizontalScrollDirection === "backward" ? Math.max(1, overscanCount) : 1;
      const overscanForward = horizontalScrollDirection === "forward" ? Math.max(1, overscanCount) : 1;

      return [Math.max(0, startIndex - overscanBackward), Math.min(columnCount - 1, stopIndex + overscanForward)];
    }

    getVerticalRangeToRender() {
      const { columnCount, overscanCount } = this.props;
      const { verticalScrollDirection, scrollTop } = this.state;

      const startIndex = getRowStartIndexForOffset(this.props, scrollTop);
      const stopIndex = getRowStopIndexForStartIndex(this.props, startIndex);

      // Overscan by one cell in each direction so that tab/focus works.
      // If there isn't at least one extra cell, tab loops back around.
      const overscanBackward = verticalScrollDirection === "backward" ? Math.max(1, overscanCount) : 1;
      const overscanForward = verticalScrollDirection === "forward" ? Math.max(1, overscanCount) : 1;

      return [Math.max(0, startIndex - overscanBackward), Math.min(columnCount - 1, stopIndex + overscanForward)];
    }

  }, _class.defaultProps = {
    overscanCount: 1,
    useIsScrolling: false
  }, _temp2;
}

const validateSharedProps = ({ children, height, width }) => {
  if (typeof children !== "function") {
    throw Error('An invalid "children" prop has been specified. ' + "Value should be a function that creates a React element. " + `"${children === null ? "null" : typeof children}" was specified.`);
  }

  if (typeof width !== "number") {
    throw Error('An invalid "width" prop has been specified. ' + "Grids must specify a number for width. " + `"${width === null ? "null" : typeof width}" was specified.`);
  }

  if (typeof height !== "number") {
    throw Error('An invalid "height" prop has been specified. ' + "Grids must specify a number for height. " + `"${height === null ? "null" : typeof height}" was specified.`);
  }
};